/*************************************************************************
**
** Definition of extended range date classe
**
** Created : 031102
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
**********************************************************************/

#ifndef EXTDATE_H
#define EXTDATE_H

#ifndef QT_H
#include "qstring.h"
#include "qnamespace.h"
#endif // QT_H


/*****************************************************************************
  ExtDate class
 *****************************************************************************/

extern	void test2_unit(int y, int m, int d);

class Q_EXPORT ExtDate
{
public:
    ExtDate() { jd = my_invalid_day; }
    ExtDate( int y, int m, int d );

    bool isNull() const { return jd == 0; }
    bool isValid() const;

    int year() const;
    int month() const;
    int day() const;
    int dayOfWeek() const;
    int dayOfYear() const;
    int daysInMonth() const;
    int daysInYear() const;
    int weekNumber( int *yearNum = 0 ) const;

#ifndef QT_NO_TEXTDATE
#ifndef QT_NO_COMPAT
    static QString monthName( int month ) { return shortMonthName( month ); }
    static QString dayName( int weekday ) { return shortDayName( weekday ); }
#endif
    static QString shortMonthName( int month );
    static QString shortDayName( int weekday );
    static QString longMonthName( int month );
    static QString longDayName( int weekday );
#endif //QT_NO_TEXTDATE
#ifndef QT_NO_TEXTSTRING
#if !defined(QT_NO_SPRINTF)
    QString toString( Qt::DateFormat f = Qt::TextDate ) const;
#endif
    QString toString( const QString& format ) const;
#endif
    bool   setYMD( int y, int m, int d );

    ExtDate  addDays( int days )		const;
    ExtDate  addMonths( int months )      const;
    ExtDate  addYears( int years )        const;
    int	   daysTo( const ExtDate & )	const;

    bool   operator==( const ExtDate &d ) const { return jd == d.jd; };
    bool   operator!=( const ExtDate &d ) const { return jd != d.jd; };
    bool   operator<( const ExtDate &d )	const { return jd < d.jd; };
    bool   operator<=( const ExtDate &d ) const { return jd <= d.jd; };
    bool   operator>( const ExtDate &d )	const { return jd > d.jd; };
    bool   operator>=( const ExtDate &d ) const { return jd >= d.jd; };

    static ExtDate currentDate( Qt::TimeSpec ts = Qt::LocalTime );
#ifndef QT_NO_DATESTRING
    static ExtDate fromString( const QString& s, Qt::DateFormat f = Qt::TextDate );
#endif
    static bool	 isValid( int y, int m, int d );
    static bool	 leapYear( int year );

    static uint	 gregorianToJulian( int y, int m, int d );
    static void	 julianToGregorian( uint jd, int &y, int &m, int &d );
private:
    int	 jd;
    void   set_jd(int the_jd) {jd = the_jd;};
    static int	month_length[12];
    static int	month_origin[12];
    static QString my_shortMonthNames[12];
    static QString my_shortDayNames[7];
    static QString my_longMonthNames[12];
    static QString my_longDayNames[7];
    QString toStringSimpleArg( char code, int nb ) const;

    static int my_invalid_day;
    friend class ExtDateTime;
    static int dayOfYear(int y, int m, int d);
    static int dayRank(int y, int m, int d);
#ifndef QT_NO_DATASTREAM
    friend Q_EXPORT QDataStream &operator<<( QDataStream &, const ExtDate & );
    friend Q_EXPORT QDataStream &operator>>( QDataStream &, ExtDate & );
#endif
};


/*****************************************************************************
  QTime class
 *****************************************************************************/

// deleted : nothing has to be changed about the time !

/*****************************************************************************
  ExtDateTime class
 *****************************************************************************/


/*****************************************************************************
  Date and time stream functions
 *****************************************************************************/

#ifndef QT_NO_DATASTREAM
Q_EXPORT QDataStream &operator<<( QDataStream &, const ExtDate & );
Q_EXPORT QDataStream &operator>>( QDataStream &, ExtDate & );
#endif // QT_NO_DATASTREAM

#endif // EXTDATE_H

